window.Gleam.initSearch([{"doc":"context_fp_gleam","title":"context_fp_gleam","content":"# context_fp_gleam\n\n## Deprecated in favor [functx](https://github.com/darky/functx)\n\n[![Package Version](https://img.shields.io/hexpm/v/context_fp_gleam)](https://hex.pm/packages/context_fp_gleam)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/context_fp_gleam/)\n\n![logo](logo.png)\n\nFunctional programming context for Gleam\n\n```sh\ngleam add context_fp_gleam\n```\n\n## Features\n\n- 💉 Dependency injection without magic, only functions\n- 🤌 Functions cached during workflow, no excess cost of CPU\n- 💡 Smart type inference via generics\n- ♻️ Unit tests friendly, feel free to pass mocked function in the Context\n\n## How to\n\n#### Basic example\n\n```gleam\nimport context_fp_gleam.{cfp2}\n\nlet positive_numbers = fn(ns, _c) { list.filter(ns, fn(n) { n &gt; 0 }) }\nlet numbers_prefix = fn(_ns, _c) { &quot;Here is numbers: &quot; }\nlet positive_numbers_as_string = {\n  use ns, prefix, _c &lt;- cfp2(positive_numbers, numbers_prefix)\n  prefix\n  &lt;&gt; list.map(ns, fn(n) { int.to_string(n) })\n  |&gt; string.join(&quot;,&quot;)\n}\n\n// Here is numbers: 7,4\npositive_numbers_as_string([-1, -5, 7, 0, 4], dict.new())\n```\n\n#### Calculations cached example\n\n```gleam\nimport context_fp_gleam.{cfp1,cfp2}\n\nlet positive_numbers = fn(ns, _c) {\n  list.filter(ns, fn(n) { n &gt; 0 }) // will be called only once\n}\nlet positive_numbers_length = {\n  use ns, _c &lt;- cfp1(positive_numbers)\n  list.length(ns)\n}\nlet positive_numbers_as_string = {\n  use ns, length, _c &lt;- cfp2(positive_numbers, positive_numbers_length)\n  list.map(ns, fn(n) { int.to_string(n) })\n  |&gt; string.join(&quot;,&quot;)\n  &lt;&gt; &quot;; length - &quot;\n  &lt;&gt; int.to_string(length)\n}\n\n// 7,4; length - 2\npositive_numbers_as_string([-1, -5, 7, 0, 4], dict.new())\n```\n\n#### Dependency injection example\n\n```gleam\nimport context_fp_gleam.{cfp1}\n\nlet fetch_user_from_db = fn(_ctx, _c) { User(&quot;Petya&quot;) }\nlet fetch_user = fn(fetch_user_mock, _c) {\n  option.unwrap(fetch_user_mock, fetch_user_from_db)()\n}\nlet hello_world_user = {\n  use user, _c &lt;- cfp1(fetch_user)\n  &quot;Hello world, &quot; &lt;&gt; user.name\n}\n\n// Hello world, Vasya\nhello_world_user(option.Some(fn() { User(&quot;Vasya&quot;) }), dict.new())\n```\n\n#### Extract cache for DB transaction example\n\n```gleam\nimport context_fp_gleam.{cfp1}\n\nlet fetch_user_from_db = fn(id, _c) {\n  case id {\n    1 -&gt; option.Some(User(1, &quot;Petya&quot;, 30))\n    _ -&gt; option.None\n  }\n}\nlet user_birthday = {\n  use user, _c &lt;- cfp1(fetch_user_from_db)\n  option.map(user, fn(user) { User(..user, age: user.age + 1) })\n}\nlet save_to_db = {\n  use _, cache &lt;- cfp1(user_birthday)\n  let db_entities = dict.values(cache)\n  let queries =\n    list.flat_map(db_entities, fn(user) {\n      case dynamic.unsafe_coerce(user) {\n        option.Some(User(_, _, _) as user) -&gt; [\n          #(&quot;update users set age = $1 where id = $2; &quot;, [user.age, user.id]),\n        ]\n        _ -&gt; []\n      }\n    })\n  let transaction_sql = case list.length(queries) &gt; 0 {\n    True -&gt;\n      list.fold(queries, &quot;begin; &quot;, fn(acc, query) { acc &lt;&gt; query.0 })\n      &lt;&gt; &quot;commit;&quot;\n    False -&gt; &quot;&quot;\n  }\n}\nsave_to_db(1, dict.new())\n```\n\n## See also\n\n- [context-fp](https://github.com/darky/context-fp) - Functional programming context for TypeScript\n- [context-fp-go](https://github.com/darky/context-fp-go) - Functional programming context for Golang\n","url":"index.html"},{"doc":"context_fp_gleam","title":"cfp1","content":"pub fn cfp1(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  fun: fn(b, Dict(Dynamic, Dynamic)) -&gt; c,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; c\n\nSynonyms:\ncontext_fp_gleam.cfp1\ncontext_fp_gleam cfp1","url":"context_fp_gleam.html#cfp1"},{"doc":"context_fp_gleam","title":"cfp2","content":"pub fn cfp2(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  dep2: fn(a, Dict(Dynamic, Dynamic)) -&gt; c,\n  fun: fn(b, c, Dict(Dynamic, Dynamic)) -&gt; d,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; d\n\nSynonyms:\ncontext_fp_gleam.cfp2\ncontext_fp_gleam cfp2","url":"context_fp_gleam.html#cfp2"},{"doc":"context_fp_gleam","title":"cfp3","content":"pub fn cfp3(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  dep2: fn(a, Dict(Dynamic, Dynamic)) -&gt; c,\n  dep3: fn(a, Dict(Dynamic, Dynamic)) -&gt; d,\n  fun: fn(b, c, d, Dict(Dynamic, Dynamic)) -&gt; e,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; e\n\nSynonyms:\ncontext_fp_gleam.cfp3\ncontext_fp_gleam cfp3","url":"context_fp_gleam.html#cfp3"},{"doc":"context_fp_gleam","title":"cfp4","content":"pub fn cfp4(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  dep2: fn(a, Dict(Dynamic, Dynamic)) -&gt; c,\n  dep3: fn(a, Dict(Dynamic, Dynamic)) -&gt; d,\n  dep4: fn(a, Dict(Dynamic, Dynamic)) -&gt; e,\n  fun: fn(b, c, d, e, Dict(Dynamic, Dynamic)) -&gt; f,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; f\n\nSynonyms:\ncontext_fp_gleam.cfp4\ncontext_fp_gleam cfp4","url":"context_fp_gleam.html#cfp4"},{"doc":"context_fp_gleam","title":"cfp5","content":"pub fn cfp5(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  dep2: fn(a, Dict(Dynamic, Dynamic)) -&gt; c,\n  dep3: fn(a, Dict(Dynamic, Dynamic)) -&gt; d,\n  dep4: fn(a, Dict(Dynamic, Dynamic)) -&gt; e,\n  dep5: fn(a, Dict(Dynamic, Dynamic)) -&gt; f,\n  fun: fn(b, c, d, e, f, Dict(Dynamic, Dynamic)) -&gt; g,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; g\n\nSynonyms:\ncontext_fp_gleam.cfp5\ncontext_fp_gleam cfp5","url":"context_fp_gleam.html#cfp5"},{"doc":"context_fp_gleam","title":"cfp6","content":"pub fn cfp6(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  dep2: fn(a, Dict(Dynamic, Dynamic)) -&gt; c,\n  dep3: fn(a, Dict(Dynamic, Dynamic)) -&gt; d,\n  dep4: fn(a, Dict(Dynamic, Dynamic)) -&gt; e,\n  dep5: fn(a, Dict(Dynamic, Dynamic)) -&gt; f,\n  dep6: fn(a, Dict(Dynamic, Dynamic)) -&gt; g,\n  fun: fn(b, c, d, e, f, g, Dict(Dynamic, Dynamic)) -&gt; h,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; h\n\nSynonyms:\ncontext_fp_gleam.cfp6\ncontext_fp_gleam cfp6","url":"context_fp_gleam.html#cfp6"},{"doc":"context_fp_gleam","title":"cfp7","content":"pub fn cfp7(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  dep2: fn(a, Dict(Dynamic, Dynamic)) -&gt; c,\n  dep3: fn(a, Dict(Dynamic, Dynamic)) -&gt; d,\n  dep4: fn(a, Dict(Dynamic, Dynamic)) -&gt; e,\n  dep5: fn(a, Dict(Dynamic, Dynamic)) -&gt; f,\n  dep6: fn(a, Dict(Dynamic, Dynamic)) -&gt; g,\n  dep7: fn(a, Dict(Dynamic, Dynamic)) -&gt; h,\n  fun: fn(b, c, d, e, f, g, h, Dict(Dynamic, Dynamic)) -&gt; i,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; i\n\nSynonyms:\ncontext_fp_gleam.cfp7\ncontext_fp_gleam cfp7","url":"context_fp_gleam.html#cfp7"},{"doc":"context_fp_gleam","title":"cfp8","content":"pub fn cfp8(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  dep2: fn(a, Dict(Dynamic, Dynamic)) -&gt; c,\n  dep3: fn(a, Dict(Dynamic, Dynamic)) -&gt; d,\n  dep4: fn(a, Dict(Dynamic, Dynamic)) -&gt; e,\n  dep5: fn(a, Dict(Dynamic, Dynamic)) -&gt; f,\n  dep6: fn(a, Dict(Dynamic, Dynamic)) -&gt; g,\n  dep7: fn(a, Dict(Dynamic, Dynamic)) -&gt; h,\n  dep8: fn(a, Dict(Dynamic, Dynamic)) -&gt; i,\n  fun: fn(b, c, d, e, f, g, h, i, Dict(Dynamic, Dynamic)) -&gt; j,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; j\n\nSynonyms:\ncontext_fp_gleam.cfp8\ncontext_fp_gleam cfp8","url":"context_fp_gleam.html#cfp8"},{"doc":"context_fp_gleam","title":"cfp9","content":"pub fn cfp9(\n  dep1: fn(a, Dict(Dynamic, Dynamic)) -&gt; b,\n  dep2: fn(a, Dict(Dynamic, Dynamic)) -&gt; c,\n  dep3: fn(a, Dict(Dynamic, Dynamic)) -&gt; d,\n  dep4: fn(a, Dict(Dynamic, Dynamic)) -&gt; e,\n  dep5: fn(a, Dict(Dynamic, Dynamic)) -&gt; f,\n  dep6: fn(a, Dict(Dynamic, Dynamic)) -&gt; g,\n  dep7: fn(a, Dict(Dynamic, Dynamic)) -&gt; h,\n  dep8: fn(a, Dict(Dynamic, Dynamic)) -&gt; i,\n  dep9: fn(a, Dict(Dynamic, Dynamic)) -&gt; j,\n  fun: fn(b, c, d, e, f, g, h, i, j, Dict(Dynamic, Dynamic)) -&gt; k,\n) -&gt; fn(a, Dict(Dynamic, Dynamic)) -&gt; k\n\nSynonyms:\ncontext_fp_gleam.cfp9\ncontext_fp_gleam cfp9","url":"context_fp_gleam.html#cfp9"},{"doc":"context_fp_gleam","title":"context_fp_gleam","content":"","url":"context_fp_gleam.html"}]);